/**
 * Copyright © 2014-2024 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES. This notice
 * may not be removed from this file.
 */

package com.compdfkitpdf.reactnative.view;

import android.content.Context;
import android.net.Uri;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.FragmentManager;
import com.compdfkit.tools.common.pdf.CPDFDocumentFragment;
import com.compdfkit.tools.common.pdf.config.CPDFConfiguration;
import com.compdfkitpdf.reactnative.util.CPDFDocumentUtil;
import com.facebook.react.uimanager.ThemedReactContext;


public class CPDFView extends FrameLayout {

  private CPDFDocumentFragment documentFragment;

  private FragmentManager fragmentManager;

  public CPDFView(@NonNull Context context) {
    super(context);
  }

  private String document;

  private String password;

  private CPDFConfiguration configuration;

  public CPDFView(@NonNull Context context,
    @Nullable AttributeSet attributeSet) {
    super(context, attributeSet);
  }

  public CPDFView(@NonNull Context context, @Nullable AttributeSet attributeSet, int i) {
    super(context, attributeSet, i);
  }

  public void setup(ThemedReactContext reactContext, FragmentManager fragmentManager) {
    this.fragmentManager = fragmentManager;
    int width = ViewGroup.LayoutParams.MATCH_PARENT;
    int height = ViewGroup.LayoutParams.MATCH_PARENT;
    ViewGroup.LayoutParams params = new ViewGroup.LayoutParams(width, height);
    setLayoutParams(params);
  }

  public void setDocument(String document, String password) {
    this.password = password;
    if (document.startsWith(CPDFDocumentUtil.ASSETS_SCHEME)) {
      this.document = CPDFDocumentUtil.getAssetsDocument(getContext(), document);
    } else if (document.startsWith(CPDFDocumentUtil.CONTENT_SCHEME)) {
      this.document = document;
    } else if (document.startsWith(CPDFDocumentUtil.FILE_SCHEME)) {
      Uri uri = Uri.parse(document);
      this.document = uri.toString();
    } else {
      this.document = document;
    }
    initDocumentFragment();
  }

  public void setConfiguration(CPDFConfiguration configuration){
    this.configuration = configuration;
    initDocumentFragment();
  }

  private void initDocumentFragment(){
    if (TextUtils.isEmpty(document) || configuration == null){
      return;
    }
    if (documentFragment == null) {
      if (document.startsWith(CPDFDocumentUtil.CONTENT_SCHEME) ||
        document.startsWith(CPDFDocumentUtil.FILE_SCHEME)){
        documentFragment = CPDFDocumentFragment.newInstance(Uri.parse(document), password, configuration);
      }else {
        documentFragment = CPDFDocumentFragment.newInstance(document, password, configuration);
      }
      prepareFragment(documentFragment, true);
    }
  }

  private void prepareFragment(CPDFDocumentFragment documentFragment, boolean attachFragment) {
    if (attachFragment) {
      fragmentManager.beginTransaction()
        .add(documentFragment, "CPDFViewRN")
        .commitNow();
      View fragmentView = documentFragment.getView();
      addView(fragmentView, ViewGroup.LayoutParams.MATCH_PARENT,
        ViewGroup.LayoutParams.MATCH_PARENT);
    }
  }

  private final Runnable mLayoutRunnable = () -> {
    measure(
      MeasureSpec.makeMeasureSpec(getWidth(), MeasureSpec.EXACTLY),
      MeasureSpec.makeMeasureSpec(getHeight(), MeasureSpec.EXACTLY));
    layout(getLeft(), getTop(), getRight(), getBottom());
  };

  @Override
  public void requestLayout() {
    super.requestLayout();
    post(mLayoutRunnable);
  }
}
