/**
 * Copyright © 2014-2024 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES. This notice
 * may not be removed from this file.
 */

package com.compdfkitpdf.reactnative.viewmanager;


import android.app.Activity;

import android.net.Uri;
import android.util.Log;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.FragmentActivity;

import com.compdfkit.tools.common.pdf.CPDFConfigurationUtils;
import com.compdfkit.tools.common.pdf.config.CPDFConfiguration;
import com.compdfkitpdf.reactnative.view.CPDFView;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.uimanager.ThemedReactContext;
import com.facebook.react.uimanager.ViewGroupManager;
import com.facebook.react.uimanager.annotations.ReactProp;
import java.util.Map;

public class CPDFViewManager extends ViewGroupManager<CPDFView> {

  private ReactApplicationContext reactContext;

  public CPDFViewManager(ReactApplicationContext context) {
    this.reactContext = context;
  }

  @NonNull
  @Override
  public String getName() {
    return "RCTCPDFReaderView";
  }

  @NonNull
  @Override
  protected CPDFView createViewInstance(@NonNull ThemedReactContext themedReactContext) {
    Activity currentActivity = themedReactContext.getCurrentActivity();
    if (currentActivity instanceof FragmentActivity fragmentActivity) {
      CPDFView pdfView = new CPDFView(fragmentActivity);
      pdfView.setup(themedReactContext, fragmentActivity.getSupportFragmentManager());

      return pdfView;
    } else {
      throw new IllegalStateException("CPDFView can only be used in FragmentActivity subclasses");
    }
  }


  @ReactProp(name = "document")
  public void setDocument(CPDFView pdfView, ReadableMap documentInfo) {
    Log.e("ComPDFKitRN", "CPDFViewManager-setDocument()");
    String document = documentInfo.getString("path");
    String password = documentInfo.getString("password");
    Log.e("ComPDFKitRN", "document:" + document);
    Log.e("ComPDFKitRN", "password:" + password);
    pdfView.setDocument(document, password);
  }

  @ReactProp(name = "configuration")
  public void setConfiguration(CPDFView pdfView, String configurationJson) {
    Log.e("ComPDFKitRN", "CPDFViewManager-setConfiguration()");
    CPDFConfiguration configuration = CPDFConfigurationUtils.fromJson(configurationJson);
    pdfView.setConfiguration(configuration);
  }

  @Nullable
  @Override
  public Map<String, Integer> getCommandsMap() {
    Log.e("ComPDFKitRN", "CPDFViewManager-getCommandsMap()");
    return super.getCommandsMap();
  }

  @Override
  public void receiveCommand(@NonNull CPDFView root, String commandId,
    @Nullable ReadableArray args) {
    super.receiveCommand(root, commandId, args);
    Log.e("ComPDFKitRN", "CPDFViewManager-receiveCommand()");

  }

  @Override
  public boolean needsCustomLayoutForChildren() {
    return true;
  }
}
